@@ -8,7 +8,7 @@
  *  The iODBC driver manager.
  *
  *  Copyright (C) 1995 by Ke Jin <kejin@empress.com>
- *  Copyright (C) 1996-2014 by OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2016 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -95,7 +95,7 @@
 
 #include <unicode.h>
 
-#if defined (__APPLE__) && !(defined (NO_FRAMEWORKS) || defined (_LP64))
+#if defined (__APPLE__) && !defined (NO_FRAMEWORKS)
 #include <Carbon/Carbon.h>
 #endif
 
@@ -117,6 +117,7 @@ extern SQLRETURN SQL_API _iodbcdm_SetConnectOption (SQLHDBC hdbc,
     SQLUSMALLINT fOption, SQLULEN vParam, SQLCHAR waMode);
 
 
+#if 0
 #define CHECK_DRVCONN_DIALBOX(path) \
   { \
     if ((handle = DLL_OPEN(path)) != NULL) \
@@ -202,6 +203,7 @@ quit:
 
   return retVal;
 }
+#endif
 
 
 #define RETURN(_ret)							\
@@ -1091,10 +1093,6 @@ _iodbcdm_driverload (
   /* This will either load the driver dll or increase its reference count */
   hdll = _iodbcdm_dllopen ((char *) path);
   
-  /* Set flag if it is safe to unload the driver after use */
-  if (unload_safe)
-    _iodbcdm_safe_unload (hdll);
-
   if (hdll == SQL_NULL_HDLL)
     {
       PUSHSYSERR (pdbc->herr, _iodbcdm_dllerror ());
@@ -1102,6 +1100,10 @@ _iodbcdm_driverload (
       return SQL_ERROR;
     }
 
+  /* Set flag if it is safe to unload the driver after use */
+  if (unload_safe)
+    _iodbcdm_safe_unload (hdll);
+
   penv = (ENV_t *) (pdbc->henv);
 
   if (penv != NULL)
@@ -2282,7 +2284,7 @@ SQLDriverConnect_Internal (
 #if (ODBCVER >= 0x300)
   GENV (genv, NULL);
 #endif
-  HDLL hdll;
+  HDLL hdll = NULL;
   SQLCHAR *drv = NULL;
   SQLCHAR drvbuf[1024];
   SQLCHAR *dsn = NULL;
@@ -2302,11 +2304,10 @@ SQLDriverConnect_Internal (
   UWORD config;
   PCONFIG pconfig = NULL;
   BOOL bCallDmDlg = FALSE;
-#if defined (__APPLE__) && !(defined (NO_FRAMEWORKS) || defined (_LP64))
-  CFStringRef libname = NULL;
+#if defined (__APPLE__) && !defined (NO_FRAMEWORKS)
   CFBundleRef bundle = NULL;
+  CFBundleRef bundle_dll = NULL;
   CFURLRef liburl = NULL;
-  char name[1024] = { 0 };
 #endif
   SQLCHAR *filedsn = NULL;
   SQLCHAR *savefile = NULL;
@@ -2576,41 +2577,44 @@ SQLDriverConnect_Internal (
           break;
 #endif
 
-      ODBC_UNLOCK (); 
-#if defined (__APPLE__) && !(defined (NO_FRAMEWORKS) || defined (_LP64))
-      bundle = CFBundleGetBundleWithIdentifier (CFSTR ("org.iodbc.core"));
-      if (bundle)
-        {
-          /* Search for the drvproxy library */
-          liburl =
-  	      CFBundleCopyResourceURL (bundle, CFSTR ("iODBCadm.bundle"),
-	      NULL, NULL);
-          if (liburl
-              && (libname =
-                  CFURLCopyFileSystemPath (liburl, kCFURLPOSIXPathStyle)))
-            {
-              CFStringGetCString (libname, name, sizeof (name),
-                kCFStringEncodingASCII);
-	      _iodbcdm_strlcat (name, "/Contents/MacOS/iODBCadm",
-		  sizeof (name));
-              hdll = _iodbcdm_dllopen (name);
-	    }
-	  if (liburl)
-	    CFRelease (liburl);
-	  if (libname)
-	    CFRelease (libname);
-	}
+      ODBC_UNLOCK ();
+#if defined (__APPLE__)
+# if !defined(NO_FRAMEWORKS)
+    bundle = CFBundleGetBundleWithIdentifier (CFSTR ("org.iodbc.core"));
+    if (bundle)
+      {
+        /* Search for the drvproxy library */
+        liburl =
+            CFBundleCopyResourceURL (bundle, CFSTR ("iODBCadm.bundle"),
+            NULL, NULL);
+          if (liburl) {
+              bundle_dll = CFBundleCreate (NULL, liburl);
+              if (bundle_dll){
+                  if (waMode != 'W')
+                      dialproc = (HPROC)CFBundleGetFunctionPointerForName (bundle_dll, CFSTR("iodbcdm_drvconn_dialbox"));
+                  else
+                      dialproc = (HPROC)CFBundleGetFunctionPointerForName (bundle_dll, CFSTR("iodbcdm_drvconn_dialboxw"));
+              }
+          }
+          
+          if (liburl)
+            CFRelease (liburl);
+      }
+      if (!bundle_dll)
+          break;
+# endif
 #else
+
       hdll = _iodbcdm_dllopen ("libiodbcadm.so.2");
-#endif
-
+            
       if (!hdll)
-	break;
-
+         break;
+            
       if (waMode != 'W')
         dialproc = _iodbcdm_dllproc (hdll, "iodbcdm_drvconn_dialbox");
       else
         dialproc = _iodbcdm_dllproc (hdll, "iodbcdm_drvconn_dialboxw");
+#endif
 
       if (dialproc == SQL_NULL_HPROC)
         {
@@ -2625,6 +2629,7 @@ SQLDriverConnect_Internal (
           fDriverCompletion,	/* type of completion */
           &config);		/* config mode */
 
+
       ODBC_LOCK ();
       fDriverCompletion = SQL_DRIVER_NOPROMPT;
 
