@@ -7,8 +7,8 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1995 Ke Jin <kejin@empress.com>
- *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1995 by Ke Jin <kejin@empress.com>
+ *  Copyright (C) 1996-2016 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -79,7 +79,7 @@
 #define	_HDBC_H
 
 #if (ODBCVER >= 0x0300)
-#include "hdesc.h"
+#include <hdesc.h>
 #endif
 
 typedef struct _drvopt
@@ -141,7 +141,6 @@ typedef struct DBC
 
     DRVOPT *drvopt;			/* Driver specific connect options */
     SQLSMALLINT err_rec;
-    DM_CONV conv;
   }
 DBC_t;
 
