@@ -7,8 +7,8 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1995 Ke Jin <kejin@empress.com>
- *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1995 by Ke Jin <kejin@empress.com>
+ *  Copyright (C) 1996-2016 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -79,11 +79,11 @@
 #define	_HENV_H
 
 #include <iodbc.h>
-#include "dlproc.h"
+#include <dlproc.h>
 
 #include <sql.h>
 #include <sqlext.h>
-#include "ithread.h"
+#include <ithread.h>
 
 
 enum odbcapi_t
@@ -120,10 +120,9 @@ typedef struct
 				      time of env creation */
     SQLINTEGER cp_match;	/* connection pool matching method */
     struct DBC *pdbc_pool;	/* connection pool */
-#endif
+#endif    
 
     SQLSMALLINT err_rec;
-    DM_CONV conv;
   }
 GENV_t;
 
@@ -142,8 +141,7 @@ typedef struct
 
 #if (ODBCVER >= 0x300)
     SQLUINTEGER dodbc_ver;	/* driver's ODBC version */
-#endif
-    DM_CONV conv;
+#endif    
   }
 ENV_t;
 
