@@ -5,7 +5,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2016 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -161,21 +161,14 @@ static LPWSTR create_driversetupw (LPCWSTR driver, LPCWSTR attrs, BOOL add, BOOL
             for (i = 0; i < dlg.Attrs_list.count; i++)
             {
                 wchar_t *val,*key;
-                int alen = 0;
                 NSDictionary *row = [dlg.Attrs_list objectAtIndex:i];
                 NSString *nskey = (NSString*)[row valueForKey:@"key"];
-
                 if ([nskey isEqualToString:@"..."] || nskey.length==0)
                     continue;
-
                 key = conv_NSString_to_wchar(nskey);
-                alen += key ? wcslen(key) : 0;
-
                 val = conv_NSString_to_wchar((NSString*)[row valueForKey:@"val"]);
-                alen += val ? wcslen(val) : 0;
-
                 cour = connstr;
-                connstr = (wchar_t*) malloc ((size + alen + 2) * sizeof(wchar_t));
+                connstr = (wchar_t*) malloc ((size + wcslen(key) + wcslen(val) + 2) * sizeof(wchar_t));
                 if (connstr)
                 {
                     memcpy (connstr, cour, size*sizeof(wchar_t));
@@ -186,7 +179,7 @@ static LPWSTR create_driversetupw (LPCWSTR driver, LPCWSTR attrs, BOOL add, BOOL
                             wcscat(connstr + size - 1, val);
                     }
                     free (cour);
-                    size += alen + 2;
+                    size += wcslen(key) + wcslen(val) + 2;
                 }
                 else
                     connstr = cour;
@@ -275,16 +268,7 @@ static LPWSTR create_driversetupw (LPCWSTR driver, LPCWSTR attrs, BOOL add, BOOL
     _dialogCode = 0;
     
     [[self window] center];  // Center the window.
-    self.window.title =
-#if defined (__x86_64__)
-	@"iODBC Data Source Administrator (Intel 64bit)";
-#elif defined (__aarch64__)
-	@"iODBC Data Source Administrator (Apple Silicon)";
-#elif defined (__i386__)
-	@"iODBC Data Source Administrator (Intel 32bit)";
-#else
-	@"iODBC Data Source Administrator";
-#endif
+    self.window.title = (sizeof(ptr)==8) ? @"iODBC Data Source Administrator  (64-Bit Edition)" : @"iODBC Data Source Administrator";
     addDSNs_to_list(FALSE, _UserDSN_ArrController);
     addDSNs_to_list(TRUE, _SysDSN_ArrController);
     [_fdsn_tableView setDoubleAction:@selector(call_FDSN_DoubleClick)];
@@ -306,7 +290,6 @@ static LPWSTR create_driversetupw (LPCWSTR driver, LPCWSTR attrs, BOOL add, BOOL
     if (isdir.boolValue==TRUE){
         NSString *cliked_dir = [dict valueForKey:@"name"];
         NSString *new_path = [NSString stringWithFormat:@"%@/%@", _cur_dir, cliked_dir];
-
         self.cur_dir = new_path;
         wchar_t *path = conv_NSString_to_wchar(_cur_dir);
         addFDSNs_to_list(_cur_dir, FALSE, _FileDSN_ArrController);
@@ -385,10 +368,8 @@ static LPWSTR create_driversetupw (LPCWSTR driver, LPCWSTR attrs, BOOL add, BOOL
     }
     else if ([identifier isEqualToString:@"filedsn"]){
         wchar_t *cur_path = conv_NSString_to_wchar(_cur_dir);
-
         addFDSNs_to_list(_cur_dir, FALSE, _FileDSN_ArrController);
         fill_dir_menu(cur_path, _popup_dir_btn);
-
         if (cur_path) free(cur_path);
     }
     else if ([identifier isEqualToString:@"drivers"]){
