@@ -7,7 +7,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2016 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -77,7 +77,7 @@
 
 #include <iodbc.h>
 #include <odbcinst.h>
-#include "unicode.h"
+#include <unicode.h>
 
 #if defined (__APPLE__) && !defined (NO_FRAMEWORKS)
 #  include <Carbon/Carbon.h>
@@ -119,7 +119,7 @@
 #define CALL_CONFIG_DSNW(path) \
     if (path) \
       { \
-	char *tmp_path = strdup ((const char *)path); \
+	char *tmp_path = strdup (path); \
 	if (tmp_path) \
 	  { \
 	    char *ptr = strstr (tmp_path, "/Contents/MacOS/"); \
@@ -185,7 +185,7 @@
 		if ((_attrs_u8 = malloc (length * UTF8_MAX_CHAR_LEN + 1)) != NULL) \
 		  { \
 		    for (ptr = lpszAttributes, ptr_u8 = _attrs_u8; *ptr; ptr += WCSLEN (ptr) + 1, ptr_u8 += STRLEN (ptr_u8) + 1) \
-		      dm_StrCopyOut2_W2A (ptr, (SQLCHAR *)ptr_u8, WCSLEN (ptr) * UTF8_MAX_CHAR_LEN, NULL); \
+		      dm_StrCopyOut2_W2A (ptr, ptr_u8, WCSLEN (ptr) * UTF8_MAX_CHAR_LEN, NULL); \
 		    *ptr_u8 = '\0'; \
 		  } \
 	      } \
