@@ -7,7 +7,7 @@
  *
  *  The iODBC driver manager.
  *
- *  Copyright (C) 1996-2023 OpenLink Software <iodbc@openlinksw.com>
+ *  Copyright (C) 1996-2016 by OpenLink Software <iodbc@openlinksw.com>
  *  All Rights Reserved.
  *
  *  This software is released under the terms of either of the following
@@ -78,7 +78,7 @@
 #include <iodbc.h>
 #include <odbcinst.h>
 #include <iodbcadm.h>
-#include "unicode.h"
+#include <unicode.h>
 
 #include "dlf.h"
 #include "inifile.h"
@@ -398,8 +398,8 @@ SQLGetTranslatorW (HWND hwnd,
 
   if (retcode == TRUE)
     {
-      dm_StrCopyOut2_U8toW ((SQLCHAR *)_name_u8, lpszName, cbNameMax, pcbNameOut);
-      dm_StrCopyOut2_U8toW ((SQLCHAR *)_path_u8, lpszPath, cbPathMax, pcbPathOut);
+      dm_StrCopyOut2_U8toW (_name_u8, lpszName, cbNameMax, pcbNameOut);
+      dm_StrCopyOut2_U8toW (_path_u8, lpszPath, cbPathMax, pcbPathOut);
     }
 
 done:
